var dbName;
var version = 1;
var db;

if (!window.indexedDB) {
        window.indexedDB = window.mozIndexedDB || window.webkitIndexedDB || window.msIndexDB;      //webkit是chrome
        window.IDBTransaction = window.IDBTransaction || window.webkitIDBTransaction || window.msIDBTransaction;
        window.IDBKeyRange = window.IDBKeyRange || window.webkitIDBKeyRange || window.msIDBKeyRange
}

function initDB(dbname) {
    if(!window.indexedDB){
        return;
    }

    dbName = dbname;

    var DBOpenRequest = window.indexedDB.open(dbName, version);    // 打开数据库
    DBOpenRequest.onerror = function(event) {   // 如果数据库打开失败
        console.log('数据库打开失败');
    };

    DBOpenRequest.onsuccess = function(event) {   // 存储数据结果
        db = DBOpenRequest.result;
    };

//数据库首次创建版本，或者window.indexedDB.open传递的新版本（版本数值要比现在的高）
    DBOpenRequest.onupgradeneeded = function(event) {
        var db = event.target.result;

        db.onerror = function(event) {
            console.log('数据库打开失败');
        };

        // 创建一个数据库存储对象
        var objectStore = db.createObjectStore(dbName, {
            keyPath: 'id',
            autoIncrement: true
        });

        // 定义存储对象的数据项
        objectStore.createIndex('id', 'id', {
            unique: true
        });
        objectStore.createIndex('msg_id', 'msg_id');
        objectStore.createIndex('type', 'type');    // 1报警、2报修、3、工单
        objectStore.createIndex('type_c', 'type_c');    // 1报警、2报修、3、工单
        objectStore.createIndex('create_at', 'create_at');
        objectStore.createIndex('content', 'content');
        objectStore.createIndex('is_read', 'is_read');    // 1已读、0未读
        objectStore.createIndex('is_deleted', 'is_deleted');    // 1删除、0未删
    };
}

var method = {
    add: function (newItem,cb) {
        if(!window.indexedDB){
            return;
        }

        var transaction = db.transaction([dbName], "readwrite");
        // 打开已经存储的数据对象
        var objectStore = transaction.objectStore(dbName);
        // 添加到数据对象中
        var objectStoreRequest = objectStore.add(newItem);
        objectStoreRequest.onsuccess = function(event) {
            if(typeof  cb === 'function'){
                cb('添加成功')
            }
        };
        objectStoreRequest.onerror = function(event) {
            if(typeof  cb === 'function'){
                cb('添加失败：'+event.target.error.name)
            }
        };
    },
    edit: function (itemId) {
        if(!window.indexedDB){
            return;
        }
        // 编辑数据
        var transaction = db.transaction([dbName], "readwrite");
        // 打开已经存储的数据对象
        var objectStore = transaction.objectStore(dbName);
        // 获取存储的对应键的存储对象
        var myIndex = objectStore.index('id');
        var objectStoreRequest = myIndex.get(parseInt(itemId));

        objectStoreRequest.onsuccess = function(event) {
            var record = objectStoreRequest.result;
            record.is_read = 1;
            objectStore.put(record);
        }
    },
    del: function (id) {
        if(!window.indexedDB){
                return;
         }

        // 打开已经存储的数据对象
        var objectStore = db.transaction([dbName], "readwrite").objectStore(dbName);
        // 直接删除
        var objectStoreRequest = objectStore.delete(id);
        // 删除成功后
        objectStoreRequest.onsuccess = function() {
            method.show();
        };
    },
    findAll: function (cb) {    //非分页
         if(!window.indexedDB){
            return;
        }
        var objectStore = db.transaction(dbName).objectStore(dbName);
        objectStore.getAll().onsuccess = function(event) {
            if(typeof  cb === 'function'){
                cb(event.target.result.length)
            }
        };
    },
    findUnreadNum:function(cb){
        if(!window.indexedDB){
            return;
        }
        var objectStore = db.transaction(dbName).objectStore(dbName);
        var index = objectStore.index("is_read");
        var singleKeyRange = IDBKeyRange.only(0);
        var rowData = [];
        index.openCursor(singleKeyRange).onsuccess = function(event) {
            var cursor = event.target.result;
            rowData.push(cursor)
            if (!cursor) {
                if(typeof  cb === 'function'){
                    cb(rowData.length)
                }
                return;
            }
            cursor.continue();
        };
    },
    findBySelect:function(start,end,type,is_read,cb){  //分页、根据类型等搜索
        if(!window.indexedDB){
                return;
         }
        var objectStore2 = db.transaction(dbName).objectStore(dbName);
        var boundKeyRange = IDBKeyRange.bound(start, end, true, false);
        var rowData = [];
        objectStore2.openCursor(boundKeyRange).onsuccess = function (event) {
            var cursor = event.target.result;
            if (!cursor) {
                if(typeof  cb === 'function'){
                    cb(rowData)
                }
                return;
            }
            if(type > -1 && is_read > -1){  //都匹配
                if(cursor.value.type == type && cursor.value.is_read == is_read){
                    rowData.unshift(cursor.value);
                }
            }else {
                if(type > -1){ //只匹配type
                    if(cursor.value.type == type){
                        rowData.unshift(cursor.value);
                    }
                }else if(is_read > -1){ //只匹配is_read
                    if(cursor.value.is_read == is_read){
                        rowData.unshift(cursor.value);
                    }
                }else { //都不匹配
                    rowData.unshift(cursor.value);
                }
            }
            cursor.continue();
        };
        objectStore2.openCursor(boundKeyRange).onerror = function (event) {
            console.log('error')
        }
    },
    findByKey:function(){
        if(!window.indexedDB){
            return;
        }
        var objectStore = db.transaction([dbName], "readwrite").objectStore(dbName);
        var boundKeyRange = IDBKeyRange.only(keywords);
        var rowData;
        objectStore.index("nickName").openCursor(boundKeyRange).onsuccess = function (event) {
            var cursor = event.target.result;
            if (!cursor) {
                if (cb) {
                    cb({
                        error: 0,
                        data : rowData
                    })
                }
                return;
            }
            rowData = cursor.value;
            cursor.continue();
        };
    },
    clearData:function(){ //TODO  清库不删库
        if(!window.indexedDB){
            return;
        }
        //删除存储空间全部记录
        var objectStore = db.transaction([dbName], "readwrite").objectStore(dbName);
        objectStore.clear();
    },
    closeDB:function () {
        if(!window.indexedDB){
            return;
        }
        db.close();
    }
};
